#include "picsyr.h"
#include "protocole.h"
#define DEBUG

// configuration materielle  : ne pas toucher a ses lignes si l'on a rien change sur la plaquette
#byte trisc=0x87
#byte portc=0x07
#byte trisd=0x88
#byte portd=0x08

// liste des fonctions :
void setSens_d(int sens);
void setSens_g(int sens);
int set_pwm_d(int pwm,int sens);
int set_pwm_g(int pwm,int sens);
void RDA_isr(); // fonction appelle par IT :stocke le caractere dans une FIFO
void processDataIn(); // traite les donnes arrivees, si une tramme est bonne elle sera traitee par processCommand, dans tous les cas, un message sera retourne.
void processCommand(); // etudie les differents parametres passes dans la tramme,verifie et met a jour les parametres demandees dans la commande .

/* registre pour le controle des moteurs :
   il y a un bit de sens  et un bit de pwm.
   MAX_PWM est la valeur maxi que peut prendre pwm.
   pwmActu_x est le registre qui indique la valeur courrante de la pwm
*/
// parametres du Systeme :
#define PWM_MAX 200 // vitesse maxi autorisee
int LeftSpeed ;
int RightSpeed;
int LeftSens ;
int RightSens;

// status
int PIC_STATUS =0; // voir dans protocole.h pour la definitions des differents bits de status
#define PIC_LM_ON (PIC_STATUS &S_LM_ON )>>5 // definit si le moteur gauche realise une action
#define PIC_RM_ON (PIC_STATUS &S_RM_ON )>>4 // definit si le moteur droit  realise une action

// compteurs
int stillAliveDefaultValue = 10 ; // valeurs par defaut pour le watchdog stillAlive
int stillAliveDefaultUnit  = _SEC;
int stillAliveCounter; // compteur stillAlive, la decrementation se fait dans la boucle de timer. a l'initialisation, le compteur est mis a la valeurt par defaut.
int execCounterValue = 0 ; // compteur pour l'execution d'une commande, il n'y a qu'un compteur pour les deux moteurs.
int execCounterUnit  = _SEC;

//////////////////////
// gestion du temps //
//////////////////////

int counter1ms;
int counter10ms;
int counter100ms;
int counter1s;
int counter1min;


#INT_RTCC
void RTCC_isr(){
counter1ms ++;
if (counter1ms == 10){
   counter10ms++;
   counter1ms=0;
   }
if (counter10ms == 10){
   counter100ms++;
   counter10ms=0;
   }
if (counter100ms == 10) {
   counter1s++;
   counter100ms=0;
   }
if (counter1s == 60) {
   counter1min++;
   counter1s=0;
 //  putc('M');
   }
// decrementation du STILL_ALIVE
if(stillAliveCounter>0){
   switch(stillAliveDefaultUnit){
      case _MSEC :
         stillAliveCounter--;
         break;
      case _10MSEC :
         if (counter1ms== 0)
            stillAliveCounter--;
         break;
      case _100MSEC :
         if (counter10ms== 0 && counter1ms== 0)
            stillAliveCounter--;
         break;
      case _SEC :
         if (counter100ms== 0 && counter10ms== 0 && counter1ms== 0)
            stillAliveCounter--;
         break;
      case _AVE :break;
   }
}
if(stillAliveCounter==0) {
   // arret des moteurs, set le StillAliveTimeout status a 1
   PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
	PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
	PIC_STATUS|=S_STILL_ALIVE_TIMEOUT;
   }

// decrementation du EXEC
if ( execCounterValue>0 ){

switch( execCounterUnit){
     case _MSEC :
         execCounterValue--;
         break;
      case _10MSEC :
         if (counter1ms== 0)
            execCounterValue--;
         break;
      case _100MSEC :
         if (counter10ms== 0&& counter1ms== 0)
            execCounterValue--;
         break;
      case _SEC : 
         if (counter100ms== 0&&counter10ms== 0 && counter1ms== 0){
//            putc(execCounterValue);
            execCounterValue--;}
         break;
      case _AVE :break;
      }
    }

if(execCounterValue==0) {// arret des moteurs
   PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
	PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
   }

}
/////////////////////////
//CONTROLE DES MOTEURS //
/////////////////////////

// setSens_x
void setSens_d(int sens){
if (sens==0)
portc&=0xDF;
else
portc|=0x20;
}

void setSens_g(int sens){
if (sens==0)
portc&=0xFE;
else
portc|=0x1;
}

// set_pwm_x
/*configure la vitesse et le sens de rotation du moteur x ( gauche ou droit)
attention!!!  il faut avoir pwmActuelle_x a 0 pour pouvoir changer le sens !!!
la fonction renvoit 0 si tout s'est bien passe et 1 si il y a eu une erreur
*/

#define sens_g ( portc & 1 )  // bit correspondant au sens du moteur gauche
#define sens_d ( (portc & 0x20 ) >> 5) // bit correspondant au sens du moteur droit
int pwmActu_d =0; // vitesse actuelle gauche
int pwmActu_g =0; // vitesse actuelle droite

int set_pwm_d(int pwm,int sens){
   // teste si les valeurs donnes sont correctes
   if ( ( pwm>PWM_MAX) || // VALEUR DE PWM trop forte
         (sens_d!=sens && pwmActu_d>0 && pwm>0) ) // changement de sens alors que moteur non arrete.
      return 1;
   else {//tout est ok on peut modifier la valeur
   pwmActu_d=pwm;
   set_pwm1_duty((int16)pwmActu_d);
   setSens_d(sens);
   return 0;
   }
}

int set_pwm_g(int pwm,int sens){
   // teste si les valeurs donnes sont correctes
   if ( ( pwm>PWM_MAX) || // VALEUR DE PWM trop forte
         (sens_g!=sens && pwmActu_g>0 && pwm>0) ) // changement de sens alors que moteur non arrete.
      return 1;
   else {//tout est ok on peut modifier la valeur
   pwmActu_g=pwm;
   set_pwm2_duty((int16)pwmActu_g);
   setSens_g(sens);
   return 0;
   }

}


///////////////////////////////////
// TRAITEMENT DES DATA ENTRANTES //
///////////////////////////////////

#define MAX_FIFO  30  // taille maxi des FIFOs
int inFifo[MAX_FIFO+1];    // registre destine a recevoir les donees entrantes
int outFifo[MAX_FIFO+1];   // registre dans lequel on stoque les donees qui seront ecrites
int inIndex =0;            // index d'ecriture dans la FIFO d'entree
int inReadIndex=0;         // index de lecture dans la fifo d'entree
int outIndex =0;           // index pour l'ecriture dans la fifo de sortie

// parametres pour discuter avec la fonction qui traite les donees


int newCommand =0; // a 1 cela signifie qu'il faut traiter les champs suivants. newCommand est remis a 0 apres
int pdiSize;
int pdiID;
int pdiCommand;
int pdiParameters[MAX_PARAMETERS];
#define ENGINE_STATUS pdiParameters[0]  // les definitions suivantes servent pour clarifier le code.
#define ENGINE_LSPEED pdiParameters[1]
#define ENGINE_RSPEED pdiParameters[2]
#define ENGINE_TVALUE pdiParameters[3]
#define ENGINE_TUNIT  pdiParameters[4]

#define RM_SELECT		(ENGINE_STATUS&0x01)
#define RM_SENS		((ENGINE_STATUS&0x02)>>1)
#define RM_OVERWRITE	((ENGINE_STATUS&0x04)>>2)
#define LM_SELECT 	((ENGINE_STATUS&0x10)>>4)
#define LM_SENS		((ENGINE_STATUS&0x20)	>>5)
#define LM_OVERWRITE	((ENGINE_STATUS&0x40)>>6)
int pdiParameterIndex;

/* //////////////////////////////
   @name RDA_isr
   @role fonction d'interruption des qu'un caractere arrive sur la liaison serie
   @do copie l'octet recu dans 'inFifo' et incremente 'inIndex'
   @note Il est obligatoire de lire l'octet recu si on veux sortir de l'interruption : pour faire cela utiliser la fonction getc();
   */
#int_RDA // cette ligne permet de specifier que la fonction qui suit est executee lors de l'arrivee d'un caractere.
void RDA_isr() {
   inFifo[inIndex]=getc();
   inIndex++ ;
   inIndex= inIndex % MAX_FIFO;
   if (inIndex == inReadIndex)
      PIC_STATUS|=S_BUFFER_OVERFLOW;
}

/*/////////////////////////////////
// 	@name processDataIn
	@role fonction qui analyse les trammes recues dans inFifo et remplit la reponse correspondante dans outFifo
	@do : quand elle est appele la fonction lit un caractere dans inFifo et decrement inIndex, si le caractere correspond a une etape de fin de tramme il remplit outFifo et 			sette outIndex. Si la tramme est correcte, il remplit le command_buffer avec les donnes en vue du traitement et sette newCommand a 1.
*/
/// parametres particuliers a cette fonction
int pdiState=0; //etat de la machine :
/* 	0 : wait START FLAG
	1 : wait SIZE
	2 : wait ID
	3 : wait command
	4 : wait parameters
*/
///////////////////////////////////
void processDataIn(){
int  fillOutFifo =0; // flag permetant de savoir si on remplit ou non le buffer de sortie .(1=oui)
int readCharacter; // le caractere lu. cette fonction ne lit qu'un caractere a chaque execution
readCharacter = inFifo[inReadIndex];
inReadIndex++;
inReadIndex= inReadIndex % MAX_FIFO;
switch(pdiState) {
	case 0 : // attente du flag start : on verifie que l'octet est bien START_FLAG, si oui on change d'etat, sinon, on reste dans l'etat et on prepare un message d'erreur.
		if (readCharacter==START_FLAG){
			pdiState =	1;
			PIC_STATUS&=~S_BAD_START;
			}
		else {
			pdiSize =0;
			pdiID=0;
			pdiCommand=0;
			PIC_STATUS|= S_BAD_START;
			fillOutFifo = 1;
		}
		break;
	case 1 :// attente de la size : on verifie que l'octet est bien dans les limites possible de la size, sinon on retoutrne a l'etape 1 et on prepare un message d'erreur.
		if (readCharacter>=2 && readCharacter<MAX_PARAMETERS+2){
			pdiState =	2;
			pdiSize =readCharacter-2;
			PIC_STATUS&= ~S_BAD_SIZE;
		}
		else {
			PIC_STATUS|= S_BAD_SIZE;
			fillOutFifo = 1;
			pdiState =	0;
		}
		break;
	case 2 :// on recupere l'ID
		pdiID=readCharacter;
		pdiState =	3;
		break;
	case 3 :// on recupere la commande, s'il n'y a pas de parametres, on prepare le message de retour et on revient a l'etape 0.
		pdiCommand=readCharacter;
		if (pdiSize >0){
			pdiState =	4;
			pdiParameterIndex =0;
		}
		else {
			fillOutFifo=1;
			newCommand=1;
			pdiState =	0;
		}
		break;
	case 4 ://on recupere les differnets parametres
		pdiParameters[pdiParameterIndex]=readCharacter;
		pdiParameterIndex++;
		pdiSize--;
		if (pdiSize ==0){ // c'etait le dernier parametre : on prepare le message de retour et on revient a l'etape 0.
			fillOutFifo=1;
			newCommand=1;
			pdiState =	0;
		}
		break;
	}

	if 	(fillOutFifo==1){
		outFifo[0]=START_FLAG;
		outFifo[1]=2;
		outFifo[2]=pdiID;
		outFifo[3]=PIC_STATUS;
		outIndex=4;
		}
	else {
		outIndex=0;
		}
}

/** /////////////////////////////////////////////////
	@ name processCommand();
	@role execute la commande enregistree dans les champs pdiXXX et modifie eventuellement ceux dans outFifo et outIndex
	@do
	@TODO : pas gere le signe pour la vitesse renvoyee
	@TODO prise en compte des vitesse 0 comme des stop
	@todo prise en compte de 2 compteurs separes pour les moteurs.
*/


processCommand(){
newCommand =0;
PIC_STATUS&=~S_UNKNOWN_COMMAND;
switch(pdiCommand) {
	case(ENGINE) :

		// verification des parametres  DE VITESSE CORRECTScorrects :
		if (ENGINE_LSPEED >PWM_MAX){ // verification vitesse moteur gauche
			ENGINE_LSPEED =PWM_MAX;
			PIC_STATUS|=S_BAD_PARAMETER;
			}
		if (ENGINE_RSPEED >PWM_MAX){ //verification vitesse moteur droit
			ENGINE_RSPEED =PWM_MAX;
			PIC_STATUS|=S_BAD_PARAMETER;
			}
		// verification du temps correct
		if (ENGINE_TUNIT   >_AVE){
			PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			PIC_STATUS|=S_BAD_PARAMETER;
			break; // on sort du case
			}
		// teste si le temps correspond a un STOP.
		if (ENGINE_TUNIT  <_AVE && ENGINE_TVALUE==0 ){ // il faut couper les moteurs concernes
			if (RM_SELECT) PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			if (LM_SELECT) PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			execCounterValue=ENGINE_TVALUE;
			execCounterUnit=ENGINE_TUNIT  ;
			break; // on sort du case
			}
		// verification du sens correct dans la commande
		if ((RM_SELECT &&  PIC_STATUS&S_RM_ON != 0 && sens_d!= RM_SENS)||  // probleme de sens sur le moteur droit
			(LM_SELECT &&  PIC_STATUS&S_LM_ON != 0 && sens_g!= LM_SENS)){  // probleme de sens sur le moteur gauche
			PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			PIC_STATUS|=S_BAD_PARAMETER;
			break; // sort du case
			}
		// verification qu'on a pas une commande sans overwrite sur un moteur en cours de fonctionnement
		if (RM_SELECT &&  PIC_STATUS&S_RM_ON != 0 && RM_OVERWRITE==0){  // commande impossible a realiser sur moteur droit.
			PIC_STATUS|=S_BAD_PARAMETER;
			}
		else if (RM_SELECT){ //on execute la commande sur le moteur concerne , on met a jour la tempo pour les deux moteurs .
			PIC_STATUS|=S_RM_ON;
			RightSpeed =ENGINE_RSPEED ;
         RightSens = RM_SENS;
			execCounterValue=ENGINE_TVALUE;
			execCounterUnit=ENGINE_TUNIT  ;
			}

		if (LM_SELECT &&  PIC_STATUS&S_LM_ON != 0 && LM_OVERWRITE==0){  // commande impossible a realiser sur moteur gauche.
			PIC_STATUS|=S_BAD_PARAMETER;
			}
		else if (LM_SELECT){ //on execute la commande sur le moteur concerne .
			PIC_STATUS|=S_LM_ON;
			LeftSpeed =ENGINE_LSPEED ;
         LeftSens = LM_SENS;
			execCounterValue=ENGINE_TVALUE;
			execCounterUnit=ENGINE_TUNIT  ;
			}
		break;
	case(STILL_ALIVE) : break; // do nothing
	case(SET_STILL_ALIVE) :
		if (pdiParameters[1]> _AVE ){ //UNIT>7 : cas impossible pour l'unite .
			PIC_STATUS|=S_BAD_PARAMETER;
			pdiParameters[1]=7;
			}
		if (pdiParameters[0]==0) // VALUE=0  on est en still alive ad vitam eternam
			pdiParameters[1]=7;
		stillAliveDefaultUnit = pdiParameters[1];
		stillAliveDefaultValue = pdiParameters[0];
		break;
	case(GET_SPEED) :
		outFifo[1]=4;
		outFifo[4]=pwmActu_g;
		outFifo[5]=pwmActu_d;
		outIndex=6;
		break;
	case(GET_EXEC_TIME) :
		outFifo[1]=4;
		outFifo[4]=execCounterValue;
		outFifo[5]=execCounterUnit;
		outIndex=6;
		break;
	default :
		PIC_STATUS|=S_UNKNOWN_COMMAND;
	}
outFifo[3]=PIC_STATUS;
stillAliveCounter = stillAliveDefaultValue;
}


// fonction qui regarde l'etat du systeme et update les parametres en consequence
int oldLeftSpeed ;
int oldRightSpeed ;

void testSystemStatus(){
   if  (PIC_LM_ON==0 )
      set_pwm_g(0,leftSens);
   else if (oldLeftSpeed != LeftSpeed)
      set_pwm_g(LeftSpeed,leftSens);
   if  (PIC_RM_ON==0 )
      set_pwm_d(0,RightSens);
   else if (oldRightSpeed != RightSpeed)
      set_pwm_d(RightSpeed,RightSens);
}




void main() {
int tmp_sc,tmp_sc2,i;
   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_CLOCK_DIV_2);
   setup_spi(FALSE);
   setup_psp(PSP_DISABLED);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_4); // IT toute les  1 ms
   //setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DIV_BY_1,49,1);
   setup_ccp1(CCP_PWM);
   setup_ccp2(CCP_PWM);
   enable_interrupts(INT_RDA);
   enable_interrupts(INT_RTCC);
   enable_interrupts(global);

trisc=0x80;
portc=0xFF;
trisd=0x00;
portd=0xFF;
printf("initialisation: le PiC fonctionne correctement\n ");
//printf("valeur de portc:%x",portc);

set_pwm1_duty(0);
set_pwm2_duty(0);
/*for ( tmp_sc=0; tmp_sc<100;tmp_sc++){
delay_ms(1000);
      putc('A');

portd=0x00;
delay_ms(1000);
portd=0xFF;
      putc('c');

}
*/



while(1){
	while(inReadIndex!=inIndex) // il y a des donnees dans la file d'attente
		processDataIn();
	if (newCommand ==1) // une nouvelle commande est a traiter
		processCommand();
   newCommand=0;
   if (outIndex!=0){   // des donnees sont a envoyer
      for (i=0;i<outIndex;i++)
      	putc(outFifo[i]);
      outIndex = 0;
   }
      testSystemStatus(); // reactualise la vitesse de moteurs suivant le status.

	}

}


///////////////////////////////////////////////// under this line : bordel ////////////////////////////

