#include "picsyr.h"
#include "protocole.h"
//#define DEBUG
/*registre pour le controle des moteurs :
   il y a un bit de sens  et un bit de pwm.
   MAX_PWM est la valeur maxi que peut prendre pwm.
   pwmActu_x est le registre qui indique la valeur courrante de la pwm
*/
#byte trisc=0x87
#byte portc=0x07
#byte trisd=0x88
#byte portd=0x08

// STATUS du PIC
int PIC_STATUS =0;
#define PWM_MAX 200
#define sens_g ( portc & 1 )
#define sens_d ( (portc & 0x20 ) >> 5)
int pwmActu_d =0;
int pwmActu_g =0;



//////////////////////////////////
/* 	@name RDA_isr
	@role fonction d'interruption des qu'un caractere arrive sur la liaison serie 
	@do copie l'octet recu dans 'inFifo' et incremente 'inIndex' 
	@note Il est obligatoire de lire l'octet recu si on veux sortir de l'interruption : pour faire cela utiliser la fonction getc();*/
#define MAX_FIFO  30
int inFifo[MAX_FIFO+1];
int outFifo[MAX_FIFO+1];

int inIndex =0;
int inReadIndex=0;
int outIndex =0;
#int_RDA
RDA_isr() {
int i;
inFifo[inIndex]=getc();
inIndex++ ; 
inIndex= inIndex % MAX_FIFO;
if (inIndex == inReadIndex)
	PIC_STATUS|=S_BUFFER_OVERFLOW;
}

/*/////////////////////////////////
// 	@name processDataIn
	@role fonction qui analyse les trammes recues dans inFifo et remplit la reponse correspondante dans outFifo
	@do : quand elle est appele la fonction lit un caractere dans inFifo et decrement inIndex, si le caractere correspond a une etape de fin de tramme il remplit outFifo et 			sette outIndex. Si la tramme est correcte, il remplit le command_buffer avec les donnes en vue du traitement et sette newCommand a 1.
*/
int pdiState=0; //etat de la machine :
/* 	0 : wait START FLAG
	1 : wait SIZE 
	2 : wait ID
	3 : wait command
	4 : wait parameters
*/
int newCommand =0;
int pdiSize;
int pdiID;
int pdiCommand;
int pdiParameters[MAX_PARAMETERS];
int pdiParameterIndex;
///////////////////////////////////	
void processDataIn(){
int  fillOutFifo =0;
int readCharacter;
readCharacter = inFifo[inReadIndex];
inReadIndex++;
inReadIndex= inReadIndex % MAX_FIFO;
switch(pdiState) {
	case 0 : // attente du flag start : on verifie que l'octet est bien le bon , si oui on change d'etat, sinon, on reste dans l'etat et on prepare un message d'erreur. 	
		if (readCharacter==START_FLAG){
			pdiState =	1;
			PIC_STATUS&=~S_BAD_START;	
			}
		else {
			pdiSize =0;
			pdiID=0;
			pdiCommand=0;
			PIC_STATUS|= S_BAD_START;	
			fillOutFifo = 1;
		}
		break;
	case 1 :// attente de la size : on verifie que l'octet est bien dans les limites possible de la size, sinon on retoutrne a l'etape 1 et on prepare un message d'erreur.
		if (readCharacter>=2 && readCharacter<MAX_PARAMETERS+2){
			pdiState =	2;
			pdiSize =readCharacter-2;
			PIC_STATUS&= ~S_BAD_SIZE;
		}
		else {
			PIC_STATUS|= S_BAD_SIZE;
			fillOutFifo = 1;
			pdiState =	0;
		}
		break;	
	case 2 :// on recupere l'ID 
		pdiID=readCharacter;
		pdiState =	3;		
		break;
	case 3 :// on recupere la commande, s'il n'y a pas de parametres, on prepare le message de retour et on revient a l'etape 0.
		pdiCommand=readCharacter;
		if (pdiSize >0){
			pdiState =	4;
			pdiParameterIndex =0;
		}
		else {
			fillOutFifo=1;
			newCommand=1;
			pdiState =	0;
		}	
		break;
	case 4 ://on recupere les differnets parametres		
		pdiParameters[pdiParameterIndex]=readCharacter;
		pdiParameterIndex++;
		pdiSize--;
		if (pdiSize ==0){ // c'etait le dernier parametre : on prepare le message de retour et on revient a l'etape 0.
			fillOutFifo=1;
			newCommand=1;
			pdiState =	0;
		}			
		break;
	}

	if 	(fillOutFifo==1){
		outFifo[0]=START_FLAG;
		outFifo[1]=2;
		outFifo[2]=pdiID;
		outFifo[3]=PIC_STATUS;
		outIndex=4;	
		}
	else {
		outIndex=0;
		}
}	 

/** /////////////////////////////////////////////////
	@ name processCommand();
	@role execute la commande enregistree dans les champs pdiXXX et modifie eventuellement ceux dans outFifo et outIndex
	@do 
	@TODO : pas gere le signe pour la vitesse renvoyee 
	@TODO prise en compte des vitesse 0 comme des stop
	@todo prise en compte de 2 compteurs separes pour les moteurs.
*/
int stillAliveCounter;
int stillAliveDefaultValue = 10 ;
int stillAliveDefaultUnit  = _SEC;
int execCounterValue = 0 ;
int execCounterUnit  = _SEC;

processCommand(){
PIC_STATUS&=~S_UNKNOWN_COMMAND;
switch(pdiCommand) {
	case(ENGINE) : 
		
		// verification des parametres  DE VITESSE CORRECTScorrects :
		if (pdiParameters[1]>PWM_MAX){ // verification vitesse moteur gauche
			pdiParameters[1]=PWM_MAX;
			PIC_STATUS|=S_BAD_PARAMETER;
			} 
		if (pdiParameters[2]>PWM_MAX){ //verification vitesse moteur droit
			pdiParameters[2]=PWM_MAX;
			PIC_STATUS|=S_BAD_PARAMETER;
			}
		// verification du temps correct
		if (pdiParameters[4] >_AVE){
			PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			PIC_STATUS|=S_BAD_PARAMETER;
			break;
			}
		// teste si le temps correspond a un STOP.
		if (pdiParameters[4]<_AVE && pdiParameters[3]==0 ){ // il faut couper les moteurs concernes
			if (RM_SELECT) PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			if (LM_SELECT) PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			execCounterValue=pdiParameters[3];
			execCounterUnit=pdiParameters[4];
			break;
			}
		// verification du sens correct dans la commande
		if ((RM_SELECT &&  PIC_STATUS&S_RM_ON != 0 && sens_d!= RM_SENS)||  // probleme de sens sur le moteur droit   
			(LM_SELECT &&  PIC_STATUS&S_LM_ON != 0 && sens_g!= LM_SENS)){  // probleme de sens sur le moteur gauche 
			PIC_STATUS&=~S_RM_ON; // desactive le moteur droit
			PIC_STATUS&=~S_LM_ON; // desactive le moteur gauche
			PIC_STATUS|=S_BAD_PARAMETER;
			break; // sort du case
			}	
		// verification qu'on a pas une commande sans overwrite sur un moteur en cours de fonctionnement
		if (RM_SELECT &&  PIC_STATUS&S_RM_ON != 0 && RM_OVERWRITE){  // commande impossible a realiser sur moteur droit.   
			PIC_STATUS|=S_BAD_PARAMETER;			
			}
		else if (RM_SELECT){ //on execute la commande sur le moteur concerne .
			PIC_STATUS|=S_RM_ON;
			pwmActu_d=pdiParameters[2];
			execCounterValue=pdiParameters[3];
			execCounterUnit=pdiParameters[4];			
			}	
		if (LM_SELECT &&  PIC_STATUS&S_LM_ON != 0 && LM_OVERWRITE){  // commande impossible a realiser sur moteur gauche.   
			PIC_STATUS|=S_BAD_PARAMETER;	
			}
		else if (LM_SELECT){ //on execute la commande sur le moteur concerne .
			PIC_STATUS|=S_LM_ON;
			pwmActu_l=pdiParameters[2];
			execCounterValue=pdiParameters[3];
			execCounterUnit=pdiParameters[4];			
			}
		break; 
	case(STILL_ALIVE) : break; // do nothing
	case(SET_STILL_ALIVE) :
		if (pdiParameters[1]> _AVE ){ //UNIT>7 : cas impossible pour l'unite .
			PIC_STATUS|=S_BAD_PARAMETER;
			pdiParameters[1]=7;
			}
		if (pdiParameters[0]==0) // VALUE=0  on est en still alive ad vitam eternam
			pdiParameters[1]=7;
		stillAliveDefaultUnit = pdiParameters[1]; 
		stillAliveDefaultValue = pdiParameters[0];
		break; 
	case(GET_SPEED) : 
		outFifo[1]=4;
		outFifo[4]=pwmActu_g;
		outFifo[5]=pwmActu_d;
		outIndex=6;	
		break; 
	case(GET_EXEC_TIME) : 
		outFifo[1]=4;
		outFifo[4]=execCounterValue;
		outFifo[5]=execCounterUnit;
		outIndex=6;	
		break; 
	default : 
		PIC_STATUS|=S_UNKNOWN_COMMAND;	
	}
outFifo[3]=PIC_STATUS;
stillAliveCounter = stillAliveDefaultUnit;


}


///////////////
// setSens_x //
///////////////
void setSens_d(sens){
if (sens==0)
portc&=0xDF;
else
portc|=0x20;
}

void setSens_g(sens){
if (sens==0)
portc&=0xFE;
else
portc|=0x1;
}

///////////////
// set_pwm_x //
///////////////
/*configure la vitesse et le sens de rotation du moteur x ( gauche ou droit)
attention!!!  il faut avoir pwmActuelle_x a 0 pour pouvoir changer le sens !!!
la valeur renvoye est 'ACK' si tout est OK
et 'IMP' is on a envoy une commande impossible a realiser
*/
int set_pwm_d(int pwm,int sens){
   // teste si les valeurs donnes sont correctes
   if ( ( pwm>PWM_MAX) || // VALEUR DE PWM trop forte
         (sens_d!=sens && pwmActu_d>0 && pwm>0) ) // changement de sens alors que moteur non arrete.
      return IMP;
   else {//tout est ok on peut modifier la valeur
   pwmActu_d=pwm;
   setPwm1Duty((int16)pwmActu_d);
   setSens_d(sens);
   return ACK;
   }
}

int set_pwm_g(int pwm,int sens){
   // teste si les valeurs donnes sont correctes

   if ( ( pwm>PWM_MAX) || // VALEUR DE PWM trop forte
         (sens_g!=sens && pwmActu_g>0 && pwm>0) ) // changement de sens alors que moteur non arrete.
      return IMP;
   else {//tout est ok on peut modifier la valeur
   pwmActu_g=pwm;
   setPwm2Duty((int16)pwmActu_g);
   setSens_g(sens);
   return ACK;
   }

}


void main() {
int tmp_sc,tmp_sc2,i;
   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_CLOCK_DIV_2);
   setup_spi(FALSE);
   setup_psp(PSP_DISABLED);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_2);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DIV_BY_1,49,1);
   setup_ccp1(CCP_PWM);
   setup_ccp2(CCP_PWM);
   enable_interrupts(INT_RDA);
   enable_interrupt(global);
   flag_busy_recu=0;
trisc=0x80;
portc=0xFF;
trisd=0x00;
portd=0xFF;
printf("initialisation: le PiC fonctionne correctement\n ");
//printf("valeur de portc:%x",portc);

setPwm1Duty(0);
setPwm2Duty(0);
/*for ( tmp_sc=0; tmp_sc<100;tmp_sc++){
delay_ms(1000);
      putc('A');

portd=0x00;
delay_ms(1000);
portd=0xFF;
      putc('c');

}
*/



while(1){
	while(inReadIndex!=inIndex) // il y a des donnees dans la file d'attente	
		processDataIn();
	if (newCommand ==1) // une nouvelle commande est a traiter
		processCommand();
	while (outIndex!=0) // des donnees sont a envoyer
		putc(outFifo[--outIndex];	
	testSystemStatus(); // s'il faut interrompre les moteur a cause du changement du status. 	

	}

}


///////////////////////////////////////////////// under this line : bordel ////////////////////////////







/*

   if (flag_recu==10){ // on a rcu un message complet : analyse
   switch (message_recu[0]){
   case DEBUG_VITESSE :
   #ifdef DEBUG
   message_recu[1]-='a';
   message_recu[1]*=10;
   printf("VITESSE=%u",message_recu[1]);
   putc(message_recu[2]);
   message_recu[2]-='0';
   #endif
      tmp_sc= set_pwm_d(message_recu[1],message_recu[2]);
      tmp_sc2 =set_pwm_g(message_recu[1],message_recu[2]);
      #ifdef DEBUG
         printf("\r\n sens_d=%u, pwmActu_d=%u\r\n",sens_d,pwmActu_d);
         printf(" sens_g=%u, pwmActu_g=%u\r\n",sens_g,pwmActu_g);
      #endif


      putc(tmp_sc);
      putc(tmp_sc2);

      flag_recu=0;
   break;
   case DEBUG_STOP :
   #ifdef DEBUG
   printf("STOP");
   #endif
      set_pwm_g(0,0);
      set_pwm_d(0,0);
      putc(ACK);
      delay_ms(100); // delay d'attente avant de recevoir une autre commande pour que les moteurs soient bien arrets
      flag_recu=0;
   break;
   case DEBUG_CORRECTION :
   #ifdef DEBUG
   printf("CORRECTION");
   message_recu[1]-='a';
   message_recu[1]*=10;
   printf("VITESSE=%u",message_recu[1]);
   putc(message_recu[2]);
   message_recu[2]-='0';

   #endif
      corr_pwm_d(message_recu[1],message_recu[2]);
      corr_pwm_g(message_recu[1],message_recu[2]);
      putc(ACK);
      flag_recu=0;
      #ifdef DEBUG
         printf("\r\n sens_d=%u, pwmActu_d=%u\r\n",sens_d,pwmActu_d);
         printf(" sens_g=%u, pwmActu_g=%u\r\n",sens_g,pwmActu_g);
      #endif

   break;
   case VITESSE :
      tmp_sc= set_pwm_d(message_recu[1],message_recu[2]);
      tmp_sc2 =set_pwm_g(message_recu[1],message_recu[2]);
      if (tmp_sc!=ACK )
            putc(tmp_sc);
      else 
         putc(tmp_sc2);
      flag_recu=0;
   break;
   case VITESSE_D :
      tmp_sc= set_pwm_d(message_recu[1],message_recu[2]);
      putc(tmp_sc);
      flag_recu=0;

   break;
   case VITESSE_G :
      tmp_sc= set_pwm_g(message_recu[1],message_recu[2]);
      putc(tmp_sc);
      flag_recu=0;

   break;
   case STOP :
      set_pwm_g(0,0);
      set_pwm_d(0,0);
      delay_ms(100); // delay d'attente avant de recevoir une autre commande pour que les moteurs soient bien arrets
      putc(ACK);
      flag_recu=0;
   break;
   case STOP_G :
      set_pwm_g(0,0);
      delay_ms(100); // delay d'attente avant de recevoir une autre commande pour que les moteurs soient bien arrets
      putc(ACK);
      flag_recu=0;
   break;
   case STOP_D :
      set_pwm_d(0,0);
      delay_ms(100); // delay d'attente avant de recevoir une autre commande pour que les moteurs soient bien arrets
      putc(ACK);
      flag_recu=0;
   break;
   case CORRECTION :
      corr_pwm_d(message_recu[1],message_recu[2]);
      corr_pwm_g(message_recu[1],message_recu[2]);
      putc(ACK);
      flag_recu=0;
   break;
   case CORRECTION_G :
      corr_pwm_g(message_recu[1],message_recu[2]);
      putc(ACK);
      flag_recu=0;
   break;
   case CORRECTION_D :
      corr_pwm_d(message_recu[1],message_recu[2]);
      putc(ACK);
      flag_recu=0;
   break;

   default :
   #ifdef DEBUG
   printf("MESSAGE NON COMPRIS");
     flag_recu=0;
   #endif
      putc(NACK);
      }
     }
   }
*/
